#!/bin/bash

PKG=$1

ulimit -S -v 16000000 || true

mkdir -p cycles

if [ ! -s /srv/jenkins-workspace/goto-cc-$PKG/goto-binaries.tar.bz2 ] ; then
  echo "No goto-binaries.tar.bz2 found for $PKG"
  exit 1
fi

if ! mkdir cycles/$PKG > /dev/null 2>&1 ; then
  echo "$PKG already done"
  exit 0
fi

tmpd=`mktemp -d --tmpdir=. i-s.XXXXXX`
cd $tmpd
tar xjf /srv/jenkins-workspace/goto-cc-$PKG/goto-binaries.tar.bz2

ifs="$IFS"
IFS="
"
find goto-binaries -type f > goto-binaries.list
while read f ; do
  if echo "$f" | grep -q '\.linked$' ; then
    continue
  # ignore files generated by dh_girepository
  elif echo "$f" | grep -q '\.typelib\.so$' ; then
    continue
  fi

  ff="`basename "$f"`"
  if [ -e ../cycles/$PKG/$ff ] || [ -e ../cycles/$PKG/$ff.failed ] ; then
    i=0
    while [ -e ../cycles/$PKG/$ff.$i ] || [ -e ../cycles/$PKG/$ff.$i.failed ] ; do
      i=$((i + 1))
    done
    ff="$ff.$i"
  fi
  
  echo -n "`date` $f"
  if ! timeout -k10 900 /srv/jenkins-workspace/JOB-prepare-binaries-master/goto-instrument-git --static-cycles "$f" > ../cycles/$PKG/$ff 2>&1 ; then
    mv ../cycles/$PKG/$ff ../cycles/$PKG/$ff.failed
    echo " FAILED"
  else
    echo " OK"
  fi
done < goto-binaries.list

IFS="$ifs"
cd ..
rm -rf $tmpd

