#!/bin/bash

MOLE_HOME=`cd \`dirname $0\` > /dev/null 2>&1 ; echo $PWD`

echo -n "Total number of object files: "
find cycles -type f | wc -l

echo -n "Total number of critical cycles: "
ncrit=`cat cycles-proc/*/critical.txt | wc -l`
echo $ncrit

echo -n "Total number of uniproc cycles: "
nup=`cat cycles-proc/*/uniproc.txt | wc -l`
echo $nup

echo -n "Total number of cycles, incl invalid ones: "
cat cycles-proc/*/all-cycles.txt | wc -l

echo -n "Unique shapes of critical cycles: "
cat cycles-proc/*/critical-classified.txt | cut -f6 -d';' | sort | uniq -c | wc -l

echo -n "Unique shapes of uniproc cycles: "
cat cycles-proc/*/uniproc-classified.txt | cut -f6 -d';' | sort | uniq -c | wc -l

echo -n "Failed classification/critical: "
cat cycles-proc/*/critical-classified.txt | grep ";unknown$" | wc -l

echo -n "Failed classification/uniproc: "
cat cycles-proc/*/uniproc-classified.txt | grep ";unknown$" | wc -l

echo -n "Total CPU time: "
cputime=0
for f in cycles/*/* ; do
  l=`tail -n 1 $f`
  if ! echo $f | grep -q '\.failed$' ; then
    if echo $l | grep -q "Time static cycle search:" ; then
      cputime=`echo "scale=3; $cputime + \`echo $l | cut -f2 -d:\`" | bc`
    fi
  elif echo $l | grep -q "^Out of memory" ; then
    cputime=`echo "scale=3; $cputime + 450" | bc`
  elif tail -n 1 $f | egrep -q "^(Number of writes:|c::)" ; then
    cputime=`echo "scale=3; $cputime + 900" | bc`
  elif tail -n 1 $f | egrep -q "^(Partial Inlining|Time alias analysis:)" ; then
    cputime=`echo "scale=3; $cputime + 900" | bc`
  fi
done
echo "${cputime}s"

echo "Top 30 shapes:"
# cat cycles-proc/*/classified.txt | sort | uniq -c | sort -rg | head -n 10 | tr 'A-Z' 'a-z' | perl -p -e 's/^\s*(\d+)\s*(\S+)/\\textsf{$2}: \$$1\$/'
tt_coords=""
tt_data=""
rm -f top-ten.txt uniproc-top.txt thinair-top.txt observation-top.txt propagation-top.txt
cat cycles-proc/*/critical-classified.txt cycles-proc/*/uniproc-classified.txt | cut -f6 -d';' | sort | uniq -c | sort -rg | head -n 30 > top-ten.txt
while read count s ; do
  echo "\\textsf{$s}: \$$count$"
  tt_coords="$tt_coords$s, "
  tt_data="$tt_data($s,$count) [$s]"
  c=1
  for p in $(<pkg-pthread) ; do
    if [ $c -gt 3 ] ; then
      break
    fi

    if [ -s cycles-proc/$p/critical-classified.txt ] ; then
      l=`cat cycles-proc/$p/critical-classified.txt | grep ";$s$" | head -n 1`
      if [ "x$l" != "x" ] ; then
        c=$((c + 1))
        ll=`echo $l | cut -f4- -d';'`
        echo -n "%% $s: $ll "
        a=`echo "$l" | $MOLE_HOME/cycle-to-axiom.pl | cut -f7 -d';'`
        echo "[$a]"
        if [ $c -eq 2 ] ; then
          echo "$s" >> $a-top.txt
        fi
        continue
      fi
    fi

    if [ -s cycles-proc/$p/uniproc-classified.txt ] ; then
      l=`cat cycles-proc/$p/uniproc-classified.txt | grep ";$s$" | head -n 1`
      [ "x$l" != "x" ] || continue
      c=$((c + 1))
      ll=`echo $l | cut -f4- -d';'`
      echo -n "%% $s: $ll "
      echo "[sc per location]"
      if [ $c -eq 2 ] ; then
        echo "$s" >> uniproc-top.txt
      fi
    fi
  done
done < top-ten.txt
tt_coords="`echo $tt_coords | sed 's/,$//'`"

cat <<EOF
\\documentclass{article}
\\usepackage{pgfplots}
\\usetikzlibrary{backgrounds}
\\tikzset{
    extra padding/.style={
        show background rectangle,
        inner frame sep=#1,
        background rectangle/.style={
            draw=none
        }
    },
    extra padding/.default=0.5cm
}
\\pgfplotsset{width=18cm}
\\pgfrealjobname{top-ten-shapes-nn}
\\begin{document}
\\beginpgfgraphicnamed{top-ten-shapes}
\\begin{tikzpicture}[font=\Large]
\\begin{axis}[
  % title={\\Large Ten most frequent shapes},
  axis y line*=none,
  axis x line=bottom,
  xbar legend,
  ymin=0, ymax=1100000,
  enlarge x limits=0.07,
  anchor=west,
  legend columns=1,
  ylabel={number of cycles},
  xlabel={pattern},
  symbolic x coords={ $tt_coords },
  %xtick=data,
  xtick=\\empty,
  %x tick label style={rotate=90,anchor=west,shift={(2em,0em)},font=\\sf},
  nodes near coords,
  every node near coord/.append style={rotate=90, anchor=west,shift={(0.5em,0em)},font=\sf\Large}
]
\\addplot[black,fill=black,bar width=1pt,ybar,point meta=explicit symbolic] plot coordinates { $tt_data };
\\end{axis}
\\end{tikzpicture}
\\endpgfgraphicnamed
\\end{document}
EOF

echo "Classic shapes:"
export nup ncrit
cat cycles-proc/*/critical-classified.txt cycles-proc/*/uniproc-classified.txt | \
  cut -f6 -d';' | sort | uniq -c | sort -rg | \
  egrep '[[:space:]](mp|sb|lb|wrc|rwc|isa2|2\+2w|wrw\+2w|r|w\+rwc|iriw|coRR|coRW1|coRW2)$' | \
  perl -n -e '/^\s*(\d+)\s*(\S+)/; print "\\textsf{$2}: \$$1\$ (\$".(($1*100)/($ENV{nup}+$ENV{ncrit}))." \\%\$)\n"'

nmax=0
varmax=0
pnmax=""
pvmax=""
for p in $(<pkg-pthread) ; do
  [ -s cycles-proc/$p/critical-classified.txt ] || continue

  n=`cat cycles-proc/$p/critical.txt | wc -l`
  if [ $n -gt $nmax ] ; then
    nmax=$n
    pnmax=$p
  fi
  v=`cat cycles-proc/$p/critical-classified.txt | cut -f6 -d';' | sort | uniq | wc -l`
  if [ $v -gt $varmax ] ; then
    varmax=$v
    pvmax=$p
  fi
done

echo "The source package with most cycles is $pnmax (\\url{http://packages.debian.org/wheezy/$pnmax}, \$$nmax\$ cycles)"
echo -n "with the most frequently occurring patterns "
cat cycles-proc/$pnmax/critical-classified.txt | cut -f6 -d';' | sort | uniq -c | sort -rg | head -n 3 | perl -p -e 's/^\s*(\d+)\s*(\S+)/\\textsf{$2} (\$$1\$)/' | tr '\n' ',' | sed 's/,$/\n/' | sed 's/,/, /g'

echo "The source package with the widest variety of cycles is $pvmax (\\url{http://packages.debian.org/wheezy/$pvmax}, \$$varmax\$ cycles);"
echo -n "its most frequent patterns are "
cat cycles-proc/$pvmax/critical-classified.txt | cut -f6 -d';' | sort | uniq -c | sort -rg | head -n 3 | perl -p -e 's/^\s*(\d+)\s*(\S+)/\\textsf{$2} (\$$1\$)/' | tr '\n' ',' | sed 's/,$/\n/' | sed 's/,/, /g'

echo "Summary per axiom"
cat cycles-proc/*/axioms.txt | egrep -v '(thinair|observation|propagation)'
a="uniproc"
aa="sc per location"
echo -n "\\textsc{$aa} & "
n=`cat cycles-proc/*/uniproc-classified.txt | wc -l`
echo -n "\$$n\$ & "
d="${a}_count"
eval $d=$n
wc -l cycles-proc/*/uniproc-classified.txt | grep uniproc-classified.txt | sort -gr | head -n 3 | perl -p -e 's/^\s*(\d+)\s*cycles-proc\/([^\/]+)\/uniproc-classified.txt/$2 (\$$1\$)/' | tr '\n' ', ' | sed 's/,$/ \\\\\n/' | sed 's/,/, /g'
for a in thinair observation propagation ; do
  if [ x$a = xthinair ] ; then
    aa="no thin air"
  else
    aa=$a
  fi
  echo -n "\\textsc{$aa} & "
  n=`( echo "0 $a" ; cat cycles-proc/*/axioms.txt ) | sort -k2 | grep -w $a | awk '{ print $1 }' | tr '\n' '+' | sed 's/+$/+0\n/' | bc`
  echo -n "\$$n\$ & "
  d="${a}_count"
  eval $d=$n
  grep -w $a cycles-proc/*/axioms.txt | sort -gr -k 2 | head -n 3 | perl -p -e 's/^cycles-proc\/([^\/]+)\/axioms.txt:\s*(\d+) .*/$1 (\$$2\$)/' | tr '\n' ', ' | sed 's/,$/ \\\\\n/' | sed 's/,/, /g'
done

make_pie()
{
  axiom=$1
  label=$2
  data=$3

cat <<EOF
\\documentclass{article}
\\usepackage{pgf-pie}
\\usetikzlibrary{backgrounds}
\\tikzset{
    extra padding/.style={
        show background rectangle,
        inner frame sep=#1,
        background rectangle/.style={
            draw=none
        }
    },
    extra padding/.default=0.5cm
}
\\pgfrealjobname{$axiom-pie-nn}
\\begin{document}
\\beginpgfgraphicnamed{$axiom-pie}
\\begin{tikzpicture}[font=\\large]
\\pie[color=white, radius=2, explode=0.2]{$data}
\\node[below] at (0,-3.2) {\\textsc{$label}};
\\end{tikzpicture}
\\endpgfgraphicnamed
\\end{document}
EOF
}

echo "Summary per shape"

for a in uniproc thinair observation propagation ; do
  shapes=""
  cf="critical-classified.txt"
  aa="$a"
  case $a in
    uniproc) aa="sc per location" ; shapes="coWW coWR coRW1 coRW2 coRR" ; cf="uniproc-classified.txt" ;;
    thinair) aa="no thin air" ; shapes="lb 3.lb 4.lb" ;;
    observation) shapes="mp wrc isa2" ;;
    propagation) shapes="sb 2+2w rwc iriw r s" ;;
  esac

  for s in $shapes ; do
    sed -i "/^$s$/d" $a-top.txt
  done

  a_data=""
  a_other="100"
  echo "$aa"
  for s in $shapes `cat $a-top.txt | head -n 3` ; do
    echo -n "\\textsf{$s} & "
    n=`cat cycles-proc/*/$cf | grep ";$s$" | wc -l`
    echo -n "\$$n\$ & "
    c="${a}_count"
    c=${!c}
    if [ x$c != x0 ] ; then
      n="`echo "scale=2; $n*100/$c" | bc`"
      n_trunc="`echo $n | sed 's/\..*$//'`"
      a_other="`echo "scale=2; $a_other - $n" | bc`"
      if [ x$n_trunc != x0 ] && [ x$n_trunc != x ] ; then
        a_data="$a_data$n_trunc/\\textsf{$s}, "
      fi
    fi
    grep ";$s$" cycles-proc/*/$cf | sed 's/:.*;//' | sort | uniq -c | sort -gr | head -n 3 | perl -p -e 's/^\s*(\d+)\s*cycles-proc\/([^\/]+)\/(critical|uniproc)-classified.txt.*/$2 (\$$1\$)/' | tr '\n' ',' | sed 's/,$/ \\\\\n/' | sed 's/,/, /g'
  done
  n_trunc="`echo $a_other | sed 's/\..*$//'`"
  if [ x$n_trunc != x0 ] && [ x$n_trunc != x ] ; then
    a_data="$a_data$n_trunc/other"
  fi
  make_pie $a "$aa" "$a_data"
done

rm -f top-ten.txt uniproc-top.txt thinair-top.txt observation-top.txt propagation-top.txt

if [ ! -s pkg-pthread-section ] ; then
  for p in $(<pkg-pthread) ; do
    echo -n "$p:"
    grep-dctrl -S -n -s Section -X $p /srv/jenkins-workspace/JOB-Debian-wheezy/Sources
  done > pkg-pthread-section
fi

echo "Summary per functionality"
for f in httpd mail games libs ; do
  echo -n "$f ($"
  grep ":$f$" pkg-pthread-section | wc -l | sed 's/$/$) \&/' | tr '\n' ' '

  pkgs="`grep ":$f$" pkg-pthread-section | cut -f1 -d:`"
  for p in $pkgs ; do
    [ -e cycles-proc/$p/critical-classified.txt ] && cat cycles-proc/$p/critical-classified.txt
  done | cut -f6 -d';' | sort | uniq -c | sort -rg | head -n 3 | perl -p -e 's/^\s*(\d+)\s*(\S+)/\\textsf{$2} (\$$1\$)/' | tr '\n' ',' | sed 's/,$/ \& /' | sed 's/,/, /g'

  for p in $pkgs ; do
    [ -s cycles-proc/$p/critical.txt ] && wc -l cycles-proc/$p/critical.txt
  done | sort -rg | head -n 3 | perl -p -e 's/^\s*(\d+)\s*cycles-proc\/([^\/]+)\/critical.txt/$2 (\$$1\$)/' | tr '\n' ',' | sed 's/,$/ \\\\\n/' | sed 's/,/, /g'
done

